﻿// Copyright (C) 2010, 2011, 2013 panacoran <panacoran@users.sourceforge.jp>
// 
// This program is part of Protra.
//
// Protra is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, see <http://www.gnu.org/licenses/>.
// 
// $Id: PtSimConfig.cs 504 2014-02-11 09:49:09Z panacoran $

using System.Drawing;
using Protra.Lib.Data;

namespace Protra.Lib.Config
{
    /// <summary>
    /// PtSimの設定を読み書きするクラス。
    /// </summary>
    public class PtSimConfig : ConfigBase
    {
        /// <summary>
        /// ウィンドウの大きさを取得または設定する。
        /// </summary>
        public Size Size { get; set; }

        /// <summary>
        /// ウィンドウの位置を取得または設定する。
        /// </summary>
        public Point Location { get; set; }

        /// <summary>
        /// ウィンドウが最大化しているかを取得または設定する。
        /// </summary>
        public bool Maximized { get; set; }

        /// <summary>
        /// 縦のスプリッターの位置を取得または設定する。
        /// </summary>
        public int VirticalSplitterDistance { get; set; }

        /// <summary>
        /// システムの実行モードの設定を取得または設定する。(廃止予定)
        /// </summary>
        public int Mode { get; set; }

        /// <summary>
        /// タイムフレームを取得または設定する。
        /// </summary>
        public TimeFrame TimeFrame { set; get; }

        /// <summary>
        /// 銘柄リストの名前の設定を取得または設定する。
        /// </summary>
        public string BrandListName { get; set; }

        /// <summary>
        /// 選択されているシステムのファイル名を取得または設定する。
        /// </summary>
        public string SystemFile { get; set; }

        /// <summary>
        /// 設定ファイルの名前を取得する。
        /// </summary>
        protected override string ConfigName
        {
            get { return "ptsim"; }
        }

        /// <summary>
        /// 廃止予定のプロパティの値を新しいプロパティに移す。
        /// </summary>
        protected override void Upgrade()
        {
            TimeFrame = (TimeFrame)Mode;
        }

        /// <summary>
        /// 新しいプロパティの値を廃止予定のプロパティに戻す。
        /// </summary>
        protected override void Downgrade()
        {
            Mode = (int)TimeFrame;
        }
    }
}
